package net.wanji.opt.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenCrossDirDTO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.GreenwaveRealtimeMapper;
import net.wanji.opt.cache.GreenCrossDirInfoCache;
import net.wanji.opt.common.CommonUtils;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dto.RingPhaseInfoDTO;
import net.wanji.opt.dto.SchemeGreenDirGreenTimeDTO;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/04/01 11:18
 * @description 海信通过reids缓存方案相位信息，设置绿波时序图
 */
@Component
@Slf4j
@Profile("test")
public class HisenseGreenChartTask {

    @Resource
    private RedisUtils redisUtils;
    @Resource
    private GreenwaveRealtimeMapper greenwaveRealtimeMapper;


    @Scheduled(fixedRate = 10 * 60 * 1000, initialDelay = 1 * 60 * 1000)
    //@Scheduled(fixedRate = 1 * 60 * 1000, initialDelay = 30 * 1000)
    public void getGreenChart() {
        try {
            Map<String, GreenwaveRealtimePO> realTimeMap = getGreenRealTimeMap();
            // 绿波路口数据
            List<GreenCrossDirDTO> greenCrossDirCache = GreenCrossDirInfoCache.greenCrossDirCache;
            List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = getSchemeGreenDirGreenTimeDTOS(greenCrossDirCache);
            if (!CollectionUtils.isEmpty(greenCrossDirCache) && !CollectionUtils.isEmpty(schemeGreenDirGreenTimeDTOS)) {
                // 将海信方案环相位转成map形式
                HashMap<String, SchemeGreenDirGreenTimeDTO> greenDirGreenTimeMap = new HashMap<>();
                for (SchemeGreenDirGreenTimeDTO dto : schemeGreenDirGreenTimeDTOS) {
                    String crossId = dto.getCrossId();
                    Integer dir = dto.getDir();
                    greenDirGreenTimeMap.put(CommonUtils.getStrJoin(crossId, String.valueOf(dir)), dto);
                }

                // 计算绿灯红灯点位
                // 1.现根据绿波长度和速度计算行程时间
                // 2.在多个周期中计算相位差绿灯时间
                // 3.从相位差绿的时间通过两个路口距离和速度计算两个路口绿灯开始时间
                Map<Integer, List<GreenCrossDirDTO>> greenMap = greenCrossDirCache.stream().collect(Collectors.groupingBy(GreenCrossDirDTO::getGreenId));
                for (Map.Entry<Integer, List<GreenCrossDirDTO>> entry : greenMap.entrySet()) {
                    Integer greenId = entry.getKey();
                    List<GreenCrossDirDTO> value = entry.getValue();
                    // 1.获取行程时间
                    GreenCrossDirDTO indexDTO = value.get(0);
                    Integer indexDir = indexDTO.getInDir();
                    Integer totalDistance = indexDTO.getTotalDistance();
                    String greenKey = CommonUtils.getStrJoin(String.valueOf(greenId), String.valueOf(indexDir));
                    GreenwaveRealtimePO greenwaveRealtimePO = realTimeMap.get(greenKey);
                    // km/h速度换成m/s
                    Double speed = greenwaveRealtimePO.getSpeed() / 3.6;
                    Double travelTime = totalDistance / speed;

                    for (GreenCrossDirDTO greenCrossDirDTO : value) {
                        String crossId = greenCrossDirDTO.getCrossId();
                        String crossKey = CommonUtils.getStrJoin(String.valueOf(crossId), String.valueOf(indexDir));
                        // 获取海信环相位信息
                        SchemeGreenDirGreenTimeDTO firstGreenTimeDTO = greenDirGreenTimeMap.get(crossKey);
                        Integer cycleLen = firstGreenTimeDTO.getCycleLen();
                        Integer offset = firstGreenTimeDTO.getOffset();
                        Integer greenEndTime = firstGreenTimeDTO.getEnd();
                        // 计算出绿灯结束时间到周期结束时间:
                        Integer redEndTime = cycleLen - greenEndTime;
                        // 通过行程时间计算周期数量 + 1
                        double cycleSum = Math.ceil(travelTime / cycleLen) + 1;
                        for (int i = 0; i < cycleSum; i++) {
                            Double cycleIndex = i * Double.valueOf(cycleLen);
                            if (redEndTime > offset) {

                            }

                        }
                    }



                }
            }
        } catch (Exception e) {
            log.error("通过海信方案处理绿波波定时任务失败：{}", e);
            //throw new RuntimeException(e);
        }
    }

    private Map<String, GreenwaveRealtimePO> getGreenRealTimeMap() {
        Map<String, GreenwaveRealtimePO> realTimeMap = new HashMap<>();
        LambdaQueryWrapper<GreenwaveRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        List<GreenwaveRealtimePO> greenwaveRealtimePOList = greenwaveRealtimeMapper.selectList(queryWrapper);
        for (GreenwaveRealtimePO greenwaveRealtimePO : greenwaveRealtimePOList) {
            Integer greenId = greenwaveRealtimePO.getGreenId();
            String roadDirection = greenwaveRealtimePO.getRoadDirection();
            Integer inDir = GreenBeltDirEnum.getInDir(roadDirection);
            realTimeMap.put(String.valueOf(greenId).concat("_").concat(String.valueOf(inDir)), greenwaveRealtimePO);
        }
        return realTimeMap;
    }

    /**
     * 通过绿波路口获取海信路口协调方向相位绿灯信息
     *
     * @return
     * @throws JsonProcessingException
     */
    private List<SchemeGreenDirGreenTimeDTO> getSchemeGreenDirGreenTimeDTOS(List<GreenCrossDirDTO> greenCrossDirCache) throws Exception {
        List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = new ArrayList<>();
        ObjectMapper mapper = JacksonUtils.getInstance();

        if (!CollectionUtils.isEmpty(greenCrossDirCache)) {
            for (GreenCrossDirDTO greenCrossDirDTO : greenCrossDirCache) {
                Integer greenId = greenCrossDirDTO.getGreenId();
                String greenCrossId = greenCrossDirDTO.getCrossId();
                String greenDirStr = greenCrossDirDTO.getGreenDir();

                // 海信环相位信息
                String str = (String) redisUtils.getHash("utc-hisense-ring-phase", greenCrossId);
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                List<RingPhaseInfoDTO> ringPhaseInfoDTOS = mapper.readValue(str, new TypeReference<List<RingPhaseInfoDTO>>() {
                });
                // 遍历绿波路口协调方向
                if (StringUtils.isNotBlank(greenDirStr)) {
                    String[] greenDirArray = greenDirStr.split(",");
                    for (String s : greenDirArray) {
                        // 协调方向
                        Integer greenDir = Integer.valueOf(s);

                        if (!CollectionUtils.isEmpty(ringPhaseInfoDTOS)) {
                            Map<Integer, List<RingPhaseInfoDTO>> ringMap = ringPhaseInfoDTOS.stream().collect(Collectors.groupingBy(RingPhaseInfoDTO::getRingNo));
                            for (Map.Entry<Integer, List<RingPhaseInfoDTO>> entry : ringMap.entrySet()) {
                                Integer ringNo = entry.getKey();
                                if (Objects.equals(0, ringNo)) {
                                    continue;
                                }
                                List<RingPhaseInfoDTO> ringPhaseList = entry.getValue();
                                List<RingPhaseInfoDTO> sort = ringPhaseList.stream().sorted(Comparator.comparingInt(RingPhaseInfoDTO::getSort)).collect(Collectors.toList());
                                Integer start = 0;
                                SchemeGreenDirGreenTimeDTO schemeGreenDirGreenTimeDTO = new SchemeGreenDirGreenTimeDTO();
                                for (RingPhaseInfoDTO ringPhaseInfoDTO : sort) {
                                    List<RingPhaseInfoDTO.DirTurn> dirTurnInfoList = ringPhaseInfoDTO.getDirTurnInfoList();
                                    if (CollectionUtils.isEmpty(dirTurnInfoList)) {
                                        continue;
                                    }
                                    Integer phaseId = ringPhaseInfoDTO.getPhaseId();
                                    Integer greenTime = ringPhaseInfoDTO.getGreenTime();
                                    Integer yellowTime = ringPhaseInfoDTO.getYellowTime();
                                    Integer redTime = ringPhaseInfoDTO.getRedTime();
                                    Integer schemeNo = ringPhaseInfoDTO.getSchemeNo();
                                    Integer cycleLen = ringPhaseInfoDTO.getCycleLen();
                                    Integer offset = ringPhaseInfoDTO.getOffset();
                                    for (RingPhaseInfoDTO.DirTurn dirTurn : dirTurnInfoList) {
                                        Integer dir = dirTurn.getDir();
                                        Integer turn = dirTurn.getTurn();
                                        // 如果路口信息与绿波协调方向相同
                                        // 如果是红灯，作为开始时间，结束时间是start + greenTime + yellowTime
                                        if (Objects.equals(dir, greenDir) && Objects.equals(2, turn)) {
                                            // 黄灯做绿灯处理
                                            schemeGreenDirGreenTimeDTO.setCrossId(greenCrossId);
                                            schemeGreenDirGreenTimeDTO.setSchemeNo(schemeNo);
                                            schemeGreenDirGreenTimeDTO.setDir(dir);
                                            schemeGreenDirGreenTimeDTO.setPhaseId(phaseId);
                                            schemeGreenDirGreenTimeDTO.setStart(start);
                                            schemeGreenDirGreenTimeDTO.setRingNo(ringNo);
                                            schemeGreenDirGreenTimeDTO.setCycleLen(cycleLen);
                                            schemeGreenDirGreenTimeDTO.setOffset(offset);
                                            Integer endTime = start + greenTime + yellowTime;
                                            schemeGreenDirGreenTimeDTO.setEnd(endTime);
                                            schemeGreenDirGreenTimeDTOS.add(schemeGreenDirGreenTimeDTO);
                                        } else { // 如果是非协调方向， 相位时间作为开始时间
                                            start += greenTime + yellowTime + redTime;
                                        }
                                        log.error("当前路口：{}， 相位：{}， 协调方向：{}，开始：{}， 结束：{}", greenCrossId, phaseId, greenDir, start, schemeGreenDirGreenTimeDTO.getEnd());
                                    }
                                }
                            }
                        }
                    }
                }
            }
            log.error("路口绿灯协调方向时间：{}", mapper.writeValueAsString(schemeGreenDirGreenTimeDTOS));
        }
        return schemeGreenDirGreenTimeDTOS;
    }

}
